@@ -52,8 +52,4 @@ class Object
     end
   end
 
-  # 'simbol' is typo
-  alias _name2simbol _name2sym
-  alias _type2simbol _type2sym
-
 end
