@@ -1,7 +1,7 @@
 #
 # encode.rb
 #
-#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
+#   Copyright (c) 1998-2001 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 #
 #   This program is free software.
 #   You can distribute/modify this program under the terms of
@@ -16,6 +16,10 @@ require 'amstd/bug'
 
 module TMail
 
+  ###
+  ### decode
+  ###
+
   class HFdecoder
 
     def initialize( ret = nil, charset = nil )
@@ -65,7 +69,10 @@ module TMail
     end
 
     alias phrase text
-    alias value text
+
+    def kv_pair( k, v )
+      @f << k << '=' << v
+    end
 
 
     private
@@ -77,6 +84,45 @@ module TMail
   end
 
 
+  ###
+  ### encode
+  ###
+
+  tspecial     = Regexp.quote( '()<>@,.;:\\"/[]?=' )
+  control      = '\x00-\x1f\x7f-\xff'
+
+  TSPECIAL     = /[#{tspecial}]/n
+  CONTROL      = /[#{control}]/n
+  INSECURE     = /[#{tspecial}#{control} ]/n
+  INSECURE_PH  = /[#{tspecial}#{control}]/n
+
+  ENCODED = /=\?[^\s?=]+\?[qb]\?[^\s?=]+\?=/i
+
+  class << self
+
+    def quote( str )
+      if INSECURE === str then
+        %Q_"#{str.gsub '"', '\\\\"'}"_
+      else
+        str
+      end
+    end
+
+    def quote_phrase( str )
+      if INSECURE_PH === str then
+        %Q_"#{str.gsub '"', '\\\\"'}"_
+      else
+        str
+      end
+    end
+
+    def encoded?( str )
+      ENCODED === str
+    end
+
+  end   # class << TMail
+
+
   class HFencoder
 
     unless defined? BENCODE_DEBUG then
@@ -98,7 +144,7 @@ module TMail
     def initialize( ret = nil, eol = nil, charset = nil, limit = nil )
       @f     = ret || ''
       @eol   = eol || "\r\n"
-      @space = ' '
+      @space = "\t"
       @limit = (limit || 72) - @eol.size
       @opt   = '-' + case charset
                      when 'e', 'j', 's' then charset
@@ -108,15 +154,15 @@ module TMail
     end
 
     def reset
-      @buf = ''
+      @text = ''
       @lwsp = ''
-      @linelen = 0
+      @curlen = 0
     end
 
 
-    ###
-    ### add
-    ###
+    #
+    # add
+    #
 
     def header_line( line )
       scanadd line
@@ -159,18 +205,28 @@ module TMail
       end
     end
 
-    def value( str )
-      str = NKF.nkf( @opt, str )
-      if CONTROL === str then
-        encodeadd str
+    def kv_pair( k, v )   ### line folding is not used yet
+      v = NKF.nkf( @opt, v )
+
+      if not TMail::INSECURE === v then
+        add_text k + '=' + v
+
+      elsif not TMail::CONTROL === v then
+        add_text k + '=' + TMail.quote(v)
+      
       else
-        add_text TMail.quote(str)
+        kv = k + '*=' + "iso-2022-jp'ja'" + vencode(v)
+        add_text kv
       end
     end
 
-    ###
-    ### terminate
-    ###
+    def vencode( str )
+      str.gsub( TMail::INSECURE ) {|s| '%%%02x' % s[0] }
+    end
+
+    #
+    # terminate
+    #
 
     def write_in
       terminate
@@ -246,7 +302,7 @@ module TMail
           concat_a_s pre, strs[ 0, pre.size ]
         end
 
-        concat_e m[0], strs[ m.begin(0)..m.end(0) ]
+        concat_e m[0], strs[ m.begin(0) ... m.end(0) ]
 
         types = m.post_match
         strs.slice! 0, m.end(0)
@@ -274,7 +330,7 @@ module TMail
         puts "strs =#{strs.inspect}"
       end
 
-      bug! @buf unless @buf.empty?
+      bug! @text unless @text.empty?
 
       chunk = ''
       strs.each_with_index do |s,i|
@@ -327,43 +383,47 @@ module TMail
     #end
 
 
-    ###
-    ### length-free buffer
-    ###
+    #
+    # length-free buffer
+    #
 
     def add_text( str )
-# puts "text: #{str.inspect}"
-      @buf << str
+      @text << str
+#puts '---- text -------------------------------------'
+#puts "+ #{str.inspect}"
+#puts "txt >>>#{@text.inspect}<<<"
     end
 
     def add_with_encode( str )
-# puts "enco: #{str.inspect}"
       s = [str].pack('m')
       s.chop!
-      @buf << MPREFIX << s << MSUFFIX
+      @text << MPREFIX << s << MSUFFIX
     end
 
     def add_lwsp( lwsp )
-# puts "lwsp: #{lwsp.inspect}"
+#puts '---- lwsp -------------------------------------'
+#puts "+ #{lwsp.inspect}"
       if restsize <= 0 then
         fold
-        flush
-      else
-        flush
-        @lwsp = lwsp
       end
+      flush
+      @lwsp = lwsp
     end
 
     def flush
-      @f << @lwsp << @buf
-      @linelen += @lwsp.size + @buf.size
-      @buf = ''
+#puts '---- flush ----'
+#puts "spc >>>#{@lwsp.inspect}<<<"
+#puts "txt >>>#{@text.inspect}<<<"
+      @f << @lwsp << @text
+      @curlen += @lwsp.size + @text.size
+      @text = ''
       @lwsp = ''
     end
 
     def fold
+#puts '---- fold ----'
       @f << @eol
-      @linelen = 0
+      @curlen = 0
       @lwsp = @space
     end
 
@@ -372,7 +432,7 @@ module TMail
     end
 
     def restsize
-      @limit - (@linelen + @lwsp.size + @buf.size)
+      @limit - (@curlen + @lwsp.size + @text.size)
     end
 
   end   # class HFencoder
