@@ -23,7 +23,7 @@ class InstallError < StandardError; end
 
 class Installer
 
-  Version   = '2.0.2'
+  Version   = '2.0.4'
   Copyright = 'Copyright (c) 2000,2001 Minero Aoki'
 
 
@@ -251,7 +251,7 @@ class Installer
 
   def do_install
     into_dir( 'bin' ) {
-      foreach_package do |targ, |
+      foreach_package do |targ, *dummy|
         install_bin
       end
     }
@@ -264,12 +264,12 @@ class Installer
       end
     }
     into_dir( 'ext' ) {
-      foreach_package do |targ, |
+      foreach_package do |targ, *dummy|
         install_so targ
       end
     }
     into_dir( 'share' ) {
-      foreach_package do |targ, |
+      foreach_package do |targ, *dummy|
         install_dat targ
       end
     }
@@ -285,7 +285,7 @@ class Installer
   end
   
   def do_dryrun
-    unless File.directory? 'tmp' then
+    unless dir? 'tmp' then
       $stderr.puts 'setup.rb: setting up temporaly environment...'
       @verbose = $DEBUG
       begin
@@ -363,7 +363,10 @@ class Installer
                      'directory to install data' ],
     'ruby-path' => [ rubypath,
                      'path',
-                     'path to ruby interpreter' ],
+                     'path to ruby for #!' ],
+    'ruby-prog' => [ rubypath,
+                     'path',
+                     'path to ruby for installation' ],
     'make-prog' => [ 'make',
                      'name',
                      'make program to compile ruby extentions' ],
@@ -413,7 +416,7 @@ class Installer
 
     packs = packages
     (@with + @without).each do |i|
-      if not packs.include? i and not File.directory? i then
+      if not packs.include? i and not dir? i then
         raise InstallError, "no such package or directory '#{i}'"
       end
     end
@@ -424,7 +427,7 @@ class Installer
     s.split(',').each do |i|
       if /[\*\?]/ === i then
         tmp = Dir.glob(i)
-        tmp.delete_if {|d| not File.directory? d }
+        tmp.delete_if {|d| not dir? d }
         if tmp.empty? then
           tmp.push i   # causes error
         else
@@ -442,7 +445,7 @@ class Installer
     ret = []
     FILETYPES.each do |type|
       next unless File.exist? type
-      foreach_record( "#{type}/PATHCONV" ) do |dir, pack, |
+      foreach_record( "#{type}/PATHCONV" ) do |dir, pack, *dummy|
         ret.push pack
       end
     end
@@ -451,14 +454,14 @@ class Installer
 
   def foreach_package
     path = {}
-    foreach_record( './PATHCONV' ) do |dir, pack, targ, topfile, |
+    foreach_record( './PATHCONV' ) do |dir, pack, targ, topfile, *dummy|
       path[dir] = [pack, targ, topfile]
     end
 
     base = File.basename( Dir.getwd )
     Dir.foreach('.') do |dir|
       next if dir[0] == ?.
-      next unless File.directory? dir
+      next unless dir? dir
 
       path[dir] or raise "abs path for package '#{dir}' not exist"
       pack, targ, topfile = path[dir]
@@ -507,11 +510,11 @@ class Installer
     tmpf = nil
     File.open( fn ) do |f|
       first = f.gets
-      return unless /\A\#\!.*ruby/ === first
+      return unless /\A\#!.*ruby/ === first
 
       tmpf = Tempfile.open( 'amsetup' )
       tmpf.puts line
-      tmpf << first unless /\A\#\!/o === first
+      tmpf << first
       f.each {|i| tmpf << i }
       tmpf.close
     end
@@ -562,9 +565,7 @@ class Installer
 
 
   def extconf
-    # system "#{@config['ruby-path']} extconf.rb"
-    restore_mkmf_env
-    load 'extconf.rb'
+    command "#{@config['ruby-prog']} extconf.rb"
   end
 
   def make
@@ -572,8 +573,7 @@ class Installer
   end
   
   def clean
-    command_f @config['make-prog'] + ' clean'
-    # command @config['make-prog'] + ' clean' if File.file? 'Makefile'
+    command @config['make-prog'] + ' clean' if File.file? 'Makefile'
   end
 
   def install_so( dir )
@@ -604,17 +604,12 @@ class Installer
   #
 
   def into_dir( libn )
-    return unless File.directory? libn
+    return unless dir? libn
     chdir( libn ) {
       yield
     }
   end
 
-  def isdir( dn )
-    mkpath dn
-    dn
-  end
-
   def chdir( dn )
     curr = Dir.pwd
     begin
@@ -625,6 +620,11 @@ class Installer
     end
   end
 
+  def isdir( dn )
+    mkpath dn
+    dn
+  end
+
   def mkpath( dname )
     $stderr.puts "mkdir -p #{dname}" if @verbose
     return if @no_harm
@@ -637,9 +637,7 @@ class Installer
     end
     dirs.each_index do |idx|
       path = dirs[0..idx].join('')
-      unless File.directory? path then
-        Dir.mkdir path
-      end
+      Dir.mkdir path unless dir? path
     end
   end
 
@@ -661,7 +659,7 @@ class Installer
     Dir.foreach('.') do |fn|
       next if fn == '.'
       next if fn == '..'
-      if File.directory? fn then
+      if dir? fn then
         verbose_off {
           rmrf fn
         }
@@ -685,7 +683,7 @@ class Installer
     $stderr.puts "install #{from} #{to}" if @verbose
     return if @no_harm
 
-    if File.directory? to then
+    if dir? to then
       to = to + '/' + File.basename(from)
     end
     str = nil
@@ -708,46 +706,17 @@ class Installer
 
   def command( str )
     $stderr.puts str if @verbose
-
-    unless ret = system( str ) then
-      raise RuntimeError, "'system #{str}' failed"
-    end
-    ret
+    system str or raise RuntimeError, "'system #{str}' failed"
   end
 
-  def command_f( str )
-    begin
-      command str
-    rescue RuntimeError
-      ;
-    end
+  def dir?( dname )
+    # for CORRUPTED windows stat()
+    File.directory? (dname[-1,1] == '/' ? dname : dname + '/')
   end
 
 end
 
 
-require 'mkmf'
-GV = %w( $CFLAGS $CPPFLAGS $DLDFLAGS $LDFLAGS $LIBPATH $LOCAL_LIBS $OBJEXT
-         $archdir $config_cache $defs $hdrdir $libdir $libs $local_flags
-         $make $null $objs $sitearchdir $sitelibdir $srcdir $topdir )
-SAVE_ENV = {}
-GV.each do |i|
-  v = eval(i)
-  SAVE_ENV[i] = v ? v.dup : v
-end
-
-def restore_mkmf_env
-  i = nil
-
-  SAVE_ENV.each do |k,v|
-    eval "#{k} = v"
-  end
-  if defined? $configure_args then
-    $configure_args.clear
-  end
-end
-
-
 if $0 == __FILE__ then
   begin
     MainInstaller = Installer.new( ARGV )
